from django.db import models
from thumbs import ImageWithThumbsField
from django.contrib.auth.models import User, Group
from tagging.fields import TagField
from tagging.models import Tag
from myfields import ISBNField
from books.fields import MultiSelectField, MultiSelectFormField

class Book(models.Model):
  title = models.CharField(max_length=255)
  isbn = models.CharField(max_length=20, help_text='ISBN Code - digits only, no dashes, etc.', unique=True)
  pages = models.CharField(max_length=100, blank=True)
  weight = models.CharField(max_length=100, blank=True)
  dimensions = models.CharField(max_length=120, blank=True, help_text='no. of pages and dimensions of book')
  published = models.DateField(blank=True)
  first_sentence = models.TextField(blank=True)
  publisher = models.CharField(max_length=255, blank=True)
  description = models.TextField(blank=True)
  owner = models.ForeignKey('Owner', blank=True)
  location = models.ForeignKey('Location', blank=True)
  tags = TagField("Tags", blank=True, help_text="Enter as many tags as you like, separated by commas")
  autotags = TagField("AutoTags", blank=True, help_text="Will be auto filled by lookup script")
  authors = models.ManyToManyField('Author')
  mainimg = ImageWithThumbsField(upload_to='images/uploads', blank=True, sizes=((125,125),(200,200)), height_field="image_height", width_field='image_width')
  added_by = models.ForeignKey(User, editable=False)
  date_added = models.DateTimeField(auto_now=True, editable=False)

  def __unicode__(self):
    return self.title

  def get_title(self):
    return self.title

class ChapterEntry(models.Model):
  title = models.TextField()
  synopsis = models.TextField(blank=True)
  link = models.URLField(max_length=255, blank=True)
  book = models.ForeignKey(Book)

  def __unicode__(self):
    return self.title

class Weblink(models.Model):
  url = models.URLField()
  description = models.CharField(max_length=255, blank=True)
  book = models.ForeignKey(Book)

  def __unicode__(self):
    return self.url

class Author(models.Model):
  name = models.CharField(max_length=255)
  bio = models.TextField(blank=True)
  def __unicode__(self):
    return self.name

class WebReview(models.Model):
  txt = models.TextField(blank=True)
  book = models.ForeignKey(Book)

  def __unicode__(self):
    return self.txt

class Summary(models.Model):
  txt = models.TextField(blank=True)
  book = models.ForeignKey(Book)

  def __unicode__(self):
    return self.txt

class RelatedBook(models.Model):
  url = models.URLField(blank=True)
  title = models.CharField(max_length=255)
  isbn = models.CharField(max_length=16, blank=True)
  book = models.ForeignKey(Book)

  def __unicode__(self):
    return self.title

class ImageUrl(models.Model):
  typ = models.CharField(max_length=255)
  url = models.URLField(blank=True)
  book = models.ForeignKey(Book)

  def __unicode__(self):
    return self.url

class Source(models.Model):
  name = models.CharField(max_length=255)
  url = models.URLField()
  book = models.ForeignKey(Book)

  def __unicode__(self):
    return self.name

class Image(models.Model):
  image = ImageWithThumbsField(upload_to='images/uploads', sizes=((125,125),(200,200)), height_field='height', width_field='width')
  book = models.ForeignKey(Book)
  typ = models.CharField(max_length=255)

  def __unicode__(self):
    return self.image

class Owner(models.Model):
  name = models.CharField(max_length=100)

  def __unicode__(self):
    return self.name

class Location(models.Model):
  name = models.CharField(max_length=255)
  code = models.CharField(max_length=10, unique=True)

  def __unicode__(self):
    return "%s (%s)" % (self.name, self.code)


##################################### CATALOGUES ######################################################


class Catalogue(models.Model):
  title = models.CharField(max_length=255)
  artist = models.CharField(max_length=255, blank=True)
  exhibition = models.CharField(max_length=255, blank=True)
  published_date = models.DateField(blank=True, verbose_name="Published on")
  condition = models.TextField(blank=True)
  description = models.TextField(blank=True)
  owner = models.ForeignKey('Owner', blank=True)
  location = models.ForeignKey(Location, blank=True)
  tags = TagField("Tags", blank=True, help_text="Enter as many tags as you like, separated by commas")
  added_by = models.ForeignKey(User, editable=False)
  date_added = models.DateTimeField(auto_now=True, editable=False)
  def __unicode__(self):
    return self.name

  class Meta:
    verbose_name_plural = "Catalogues"


##################################### HARDWARE ######################################################

HARDWARE_CATEGORY_CHOICES = (
    ('artworks', 'Artworks'),
    ('control', 'Control'),
    ('cables', 'Cables'),
    ('connectors', 'Connectors'),
    ('computer', 'Computer'),
    ('storage', 'Storage Devices'),
    ('av', 'Audio / Video Equipment'),
    ('electronics', 'Electronics'),
    ('electrical', 'Electrical & Lighting'),
    ('mechanical', 'Mechanical Parts'),
    ('media', 'Media'),
    ('tools', 'Tools'),
    ('supplies', 'Supplies'),
    ('other', 'Other'),
)

class Box(models.Model):
  box_no = models.CharField(max_length=255, blank=True, unique=True)
  description = models.TextField(blank=True)
  owner = models.ForeignKey('Owner', blank=True, null=True)
  location = models.ForeignKey(Location, blank=True, null=True)
  tags = TagField("Tags", blank=True, help_text="Enter as many tags as you like, separated by commas")

  def __unicode__(self):
    s = ''
    if self.location:
      s += self.location.code + "/"
    s += self.box_no
    return s

  class Meta:
    verbose_name_plural = "Boxes"

class Hardware(models.Model):
  title = models.CharField(max_length=255, verbose_name="Title / Name")
  category = models.ForeignKey("HardwareCategory", blank=True, null=True)
#  category = MultiSelectField(max_length=355, choices=HARDWARE_CATEGORY_CHOICES, verbose_name="Category")
  model_number = models.CharField(max_length=255, blank=True)
  serial_number = models.CharField(max_length=255, blank=True)
  description = models.TextField(blank=True)
  quantity = models.IntegerField(blank=True, null=True, default=1, help_text="This will NOT create separate entries for the item. Use 'Save as New' to duplicate items")  
#  location = models.ForeignKey(Location, blank=True)
  tags = TagField("Tags", blank=True, help_text="Enter as many tags as you like, separated by commas")
#  added_by = models.ForeignKey(User, editable=False)
  date_added = models.DateTimeField(auto_now=True, editable=False)
  links = models.ManyToManyField('HardwareLink', blank=True, null=True)
  box = models.ForeignKey('Box', blank=True, null=True)
  published = models.BooleanField(default=False)
  image = models.ImageField(upload_to='hw_images/', blank=True, null=True)

  def __unicode__(self):
    return self.labelID()

  def labelID(self):
    s = ''
    if self.box:
      try:
        loc = Location.objects.get(pk=self.box.id)
        s += loc.code + "/"
      except:
        s += ""
      s += self.box.box_no + "/"
  #  if obj.box.location:
  #    s += obj.box.location.code + "/"
    s += str(self.id)      
    return s

  class Meta:
    verbose_name_plural = "Hardware"

class HardwareCategory(models.Model):
  name = models.CharField(max_length=255)

  def __unicode__(self):
    return self.name

  class Meta:
    verbose_name = "Hardware Category"
    verbose_name_plural = "Hardware Categories"

class HardwareLink(models.Model):
  name = models.CharField(max_length=255)
  url = models.URLField()

  def __unicode__(self):
    return self.name



